@@ -13,7 +13,7 @@ __PACKAGE__->mk_accessors(
         _bpp _true_colour _big_endian _image_format
         )
 );
-our $VERSION = '0.37';
+our $VERSION = '0.38';
 
 my $MAX_PROTOCOL_VERSION = 'RFB 003.008' . chr(0x0a);  # Max version supported
 
@@ -157,7 +157,7 @@ sub login {
         PeerAddr => $hostname || 'localhost',
         PeerPort => $port     || '5900',
         Proto    => 'tcp',
-    ) || die "Error connecting to $hostname: $!";
+    ) || die "Error connecting to $hostname: $@";
     $socket->timeout(15);
     $self->socket($socket);
 
@@ -495,6 +495,49 @@ sub _image_plus_cursor {
     return $image;
 }
 
+sub _send_key_event {
+    my ( $self, $down_flag, $key ) = @_;
+
+# A key press or release. Down-flag is non-zero (true) if the key is now pressed, zero
+# (false) if it is now released. The key itself is specified using the “keysym” values
+# defined by the X Window System.
+
+    my $socket = $self->socket;
+    $socket->print(
+        pack(
+            'CCnN',
+            4,             # message_type
+            $down_flag,    # down-flag
+            0,             # padding
+            $key,          # key
+        )
+    );
+}
+
+sub send_key_event_down {
+    my ( $self, $key ) = @_;
+    $self->_send_key_event( 1, $key );
+}
+
+sub send_key_event_up {
+    my ( $self, $key ) = @_;
+    $self->_send_key_event( 0, $key );
+}
+
+sub send_key_event {
+    my ( $self, $key ) = @_;
+    $self->send_key_event_down($key);
+    $self->send_key_event_up($key);
+}
+
+sub send_key_event_string {
+    my ( $self, $string ) = @_;
+    foreach my $key ( map {ord} split //, $string ) {
+        warn $key;
+        $self->send_key_event($key);
+    }
+}
+
 sub _send_pointer_event {
     my $self = shift;
 
@@ -1077,6 +1120,60 @@ Returns a list of encoding number/encoding name pairs.  This can be used as a cl
 
    my %encodings = Net::VNC->list_encodings();
 
+=head2 send_key_event_down
+
+Send a key down event. The keys are the same as the
+corresponding ASCII value. Other common keys:
+
+  BackSpace 0xff08
+  Tab 0xff09
+  Return or Enter 0xff0d
+  Escape 0xff1b
+  Insert 0xff63
+  Delete 0xffff
+  Home 0xff50
+  End 0xff57
+  Page Up 0xff55
+  Page Down 0xff56
+  Left 0xff51
+  Up 0xff52
+  Right 0xff53
+  Down 0xff54
+  F1 0xffbe
+  F2 0xffbf
+  F3 0xffc0
+  F4 0xffc1
+  ... ...
+  F12 0xffc9
+  Shift (left) 0xffe1
+  Shift (right) 0xffe2
+  Control (left) 0xffe3
+  Control (right) 0xffe4
+  Meta (left) 0xffe7
+  Meta (right) 0xffe8
+  Alt (left) 0xffe9
+  Alt (right) 0xffea
+
+  $vnc->send_key_event_down('A');
+
+=head2 send_key_event_up
+
+Send a key up event:
+
+  $vnc->send_key_event_up('A');
+
+=head2 send_key_event
+
+Send a key down event followed by a key up event:
+
+  $vnc->send_key_event('A');
+
+=head2 send_key_event_string
+
+Send key events for every character in a string:
+
+  $vnc->send_key_event_string('Hello');
+
 =head1 BUGS AND LIMITATIONS
 
 =head2 Bit depth
